<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\ClassStudent;
use App\Models\Attendance;
use Carbon\Carbon;

class StudentController extends Controller
{
    //this function for get class by student
    public function StudentClass(Request $request, $std_id=null){
        $classes = null;
        if($std_id)
            $classes = ClassStudent::orderBy('id', 'DESC')->with(['user', 'class'])->where('std_id', $std_id)->get();
        else
            $classes = ClassStudent::orderBy('id', 'DESC')->with(['user', 'class'])->get();

        return response()->json($classes);
    }

    //this function for get studentss by class
    public function ClassStudents(Request $request, $class_id=null){
        $students = null;
        if($class_id)
            $students = ClassStudent::orderBy('id', 'DESC')->with(['user', 'class'])->where('class_id', $class_id)->get();
        else
            $students = ClassStudent::orderBy('id', 'DESC')->with(['user', 'class'])->get();

        return response()->json($students);
    }

    //thid mthod for get daily attendance
    public function DailyAttendance(Request $request,  $class_id){
        $students = null;
        if($class_id)
            $students = ClassStudent::orderBy('id', 'DESC')->with(['user', 'class'])->where('class_id', $class_id)->get();
        $from_date = date('Y-m-d ', time()).'0:0:0';
        $to_date = date('Y-m-d ', time()).'23:59:59';

        $sheet = [];

        if($students){
            foreach($students as $student){
                $name = $student->user->name;
                $roll = $student->user->studentId;
                $present = null;
                $attendance = Attendance::whereBetween('created_at', [$from_date, $to_date])->where(['student_id'=>$student->std_id, 'class_id'=>$student->class_id])->first();
                
                if($attendance and $attendance->atd_status=='P'){
                    $present = 'P';
                }else{
                    $present = 'A';
                }
                
                $sheet[] = ['name'=>$name, 'roll'=>$roll, 'status'=>$present];
            }
        }
        return response()->json($sheet);
    }

    public function MonthlyAttendance(Request $request,  $class_id, $year, $month){
        $students = null;
        if($class_id)
            $students = ClassStudent::orderBy('id', 'DESC')->with(['user', 'class'])->where('class_id', $class_id)->get();
        

        $headings = [];
        $headings['heading'] = ['Name', 'Roll No'];
        $sheet = [];
        $dateStatus = false;
        if($students){
            foreach($students as $student){
                $name = $student->user->name;
                $roll = $student->user->studentId;
                $present = null;

                $sheet[$student->std_id] = ['name'=>$name, 'roll'=>$roll];
                
                for($i=1; $i<=31; $i++){
                    $from_date = $year.'-'.$month.'-'.$i.' 0:0:0';
                    $to_date = $year.'-'.$month.'-'.$i.' 23:59:59';

                    $attendance = Attendance::whereBetween('created_at', [$from_date, $to_date])->where(['student_id'=>$student->std_id, 'class_id'=>$student->class_id])->first();

                    if($attendance)
                        $dateStatus = true;

                    if($attendance and $attendance->atd_status=='P'){
                        $present = 'P';
                    }else{
                        $present = 'A';
                    }

                    
                    
                    $n_date = date('m/d', strtotime($from_date));
                    $headings['heading'][] = $n_date;
                    $sheet[$student->std_id][$n_date] = $present;
                }
                
                

            }
            
        }
        $n_data['heading'] = $headings['heading'];
        $n_data['attendance'] = $sheet;
        
        $output = $this->filter($class_id, $year, $month, $n_data);
        return response()->json($output);
    }


    private function filter($class_id, $year, $month, $data){
        $headings = $data['heading'];
        $attendance = $data['attendance'];

        $new_heading = ['Name', 'Roll No'];
        $new_attendance = [];

        for($i=1; $i<=31; $i++){
            $from_date = $year.'-'.$month.'-'.$i.' 0:0:0';
            $to_date = $year.'-'.$month.'-'.$i.' 23:59:59';

            $attendance_check = Attendance::whereBetween('created_at', [$from_date, $to_date])->where(['class_id'=>$class_id])->first();

            
            $n_date = date('m/d', strtotime($from_date));

            if($attendance_check){
                if(in_array($n_date, $headings)){
                    $new_heading[] = $n_date;
                }
               
                foreach($attendance as $key=>$value){
                    $new_attendance[$key]['name'] = $value['name'];
                    $new_attendance[$key]['roll'] = $value['roll'];
                    $new_attendance[$key][$n_date] = $value[$n_date];
                }
                
            }
           
        }
       

        $n_data['heading'] = $new_heading;
        $n_data['attendance'] = $new_attendance;

        return $n_data;
    }

    //this method for add student to class
    public function JoinClass(Request $request){
        $class_id = $request->input('class_id');
        $std_id = $request->input('std_id');
        $status = 'PENDING';
        $insertAttr = [
            'created_at'=>Carbon::now(),
            'class_id'=>$class_id,
            'std_id'=>$std_id,
            'status'=>$status,
        ];

        $data = [];
        if($insert_id = ClassStudent::insertGetId($insertAttr)){
            $data['status'] = 'SUCCESS';
            $data['class'] = ClassStudent::where('id', $insert_id)->first();
        }else{
            $data['status'] = 'FAIELD';
            $data['class'] =  null;
        }

        return response()->json($data);
    }

    public function UpdateJoin(Request $request, $join_id){
        $status = $request->input('status');
        $insertAttr = [
            'updated_at'=>Carbon::now(),
            'status'=>$status,
        ];

        $data = [];
        if(ClassStudent::where('id', $join_id)->update($insertAttr)){
            $data['status'] = 'SUCCESS';
            $data['class'] = ClassStudent::where('id', $join_id)->first();
        }else{
            $data['status'] = 'FAIELD';
            $data['class'] =  null;
        }
        return response()->json($data);
    }


    //this method for attendance
    public function Attendance(Request $request){
        $std_id = $request->input('std_id');
        $class_id = $request->input('class_id');
        $letval = $request->input('letval');
        $lengval = $request->input('lengval');
        
        $from_date = date('Y-m-d ', time()).'0:0:0';
        $to_date = date('Y-m-d ', time()).'23:59:59';
        $attendance = Attendance::whereBetween('created_at', [$from_date, $to_date])->where(['student_id'=>$std_id, 'class_id'=>$class_id])->first();

        $data = [];
        if(!$attendance){
            $insertAttr = [
                'student_id' => $std_id,
                'class_id' => $class_id,
                'letval' => $letval,
                'lengval' => $lengval,
                'atd_status' => 'P',
                'status' => 'APPROVED',
                'created_at' => Carbon::now(),
            ];
            if($insert_id = Attendance::insertGetId($insertAttr)){
                $data['status'] = 'SUCCESS';
                $data['attendance'] = Attendance::where('id', $insert_id)->first();
            }else{
                $data['status'] = 'FAIELD';
                $data['attendance'] = NULL;
            }
        }else{
            $data['status'] = 'DONEBEFORE';
            $data['attendance'] = $attendance;
        }

        return response()->json($data);
    }
}
